package java.util;

import java.util.function.Supplier;
import java.util.function.LongSupplier;
import java.util.function.LongConsumer;

public final class OptionalLong
{
    private static final OptionalLong EMPTY;
    private final boolean isPresent;
    private final long value;
    
    private OptionalLong() {
        this.isPresent = false;
        this.value = 0L;
    }
    
    public static OptionalLong empty() {
        return OptionalLong.EMPTY;
    }
    
    private OptionalLong(long value) {
        this.isPresent = true;
        this.value = value;
    }
    
    public static OptionalLong of(long n) {
        return new OptionalLong(n);
    }
    
    public long getAsLong() {
        if (!this.isPresent) {
            throw new NoSuchElementException("No value present");
        }
        return this.value;
    }
    
    public boolean isPresent() {
        return this.isPresent;
    }
    
    public void ifPresent(LongConsumer longConsumer) {
        if (this.isPresent) {
            longConsumer.accept(this.value);
        }
    }
    
    public long orElse(long n) {
        return this.isPresent ? this.value : n;
    }
    
    public long orElseGet(LongSupplier longSupplier) {
        return this.isPresent ? this.value : longSupplier.getAsLong();
    }
    
    public <X extends Throwable> long orElseThrow(Supplier<X> supplier) throws X, Throwable {
        if (this.isPresent) {
            return this.value;
        }
        throw (Throwable)supplier.get();
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof OptionalLong)) {
            return false;
        }
        OptionalLong optionalLong = (OptionalLong)o;
        return (this.isPresent && optionalLong.isPresent) ? (this.value == optionalLong.value) : (this.isPresent == optionalLong.isPresent);
    }
    
    @Override
    public int hashCode() {
        return this.isPresent ? Long.hashCode(this.value) : 0;
    }
    
    @Override
    public String toString() {
        return this.isPresent ? String.format("OptionalLong[%s]", new Object[] { Long.valueOf(this.value) }) : "OptionalLong.empty";
    }
    
    static {
        EMPTY = new OptionalLong();
    }
}
